<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

class database
{

    private $conn;

    private $result;

    public function connect_to_database($dbHost, $dbUser, $dbPassword, $dbName, $dbPort)
    {
        $this->conn = new MySQLI($dbHost, $dbUser, $dbPassword, $dbName, $dbPort);
        if (mysqli_connect_errno()) {
            throw new Exception("Unable to connect to database: " . mysqli_connect_error());
        }
    }

    public function query($query)
    {
        if (! $this->result = $this->conn->query($query)) {
            return false;
        }
        return true;
    }

    public function error()
    {
        return $this->conn->error;
    }

    public function create_user_table(){
        $query = "CREATE  TABLE IF NOT EXISTS `user` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `username` VARCHAR(255) NOT NULL ,
            `password` varchar(255) NOT NULL,
            UNIQUE KEY `username_UNIQUE` ( `username` ),
            PRIMARY KEY (`id`) ) ENGINE = InnoDB";
        
        return $this->query($query);
    }
    
    public function addUser($user, $password)
    {      
        $enc_password = password_hash($password, PASSWORD_DEFAULT);
        $user = $this->conn->real_escape_string($user);
        $enc_password = $this->conn->real_escape_string($enc_password);
        $query = "INSERT INTO user (username, password) VALUES ('$user','$enc_password')";
        return $this->query($query);
    }
    
    public function addAdmin($user)
    {
        $user = $this->conn->real_escape_string($user);
        $query = "INSERT INTO admin (username) VALUES ('$user')";
        return $this->query($query);
    }

    public function create_admin_table()
    {
        $query = "CREATE  TABLE IF NOT EXISTS `admin` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `username` VARCHAR(255) NOT NULL ,
            UNIQUE KEY `username_UNIQUE` ( `username` ),
            PRIMARY KEY (`id`) ) ENGINE = InnoDB";

        return $this->query($query);
    }

    public function create_queue_admin_table()
    {
        $query = "CREATE TABLE IF NOT EXISTS `queue_admin` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `queue_id` INT NULL ,
            `username` VARCHAR(64) NOT NULL,
            PRIMARY KEY (`id`),
            INDEX `queue_id` (`queue_id` ASC) ,
            INDEX `username` (`username` ASC) ,
            UNIQUE KEY `queue_user_UNIQUE` (`queue_id`,`username` ),
            CONSTRAINT `queue_id`
            FOREIGN KEY (`queue_id` )
                REFERENCES `queue` (`id` )
                ON DELETE CASCADE
                ON UPDATE CASCADE
            ) ENGINE = InnoDB";

        return $this->query($query);
    }

    public function create_queue_table()
    {
        $query = "CREATE TABLE IF NOT EXISTS `queue` (
            `id` INT NOT NULL AUTO_INCREMENT ,
            `queue_name` VARCHAR(64) NOT NULL ,
            `last_checked_date` INT  ,
            `enabled_queue` TINYINT(1)  NOT NULL ,
            PRIMARY KEY (`id`),
            UNIQUE INDEX `queue_name_UNIQUE` (`queue_name` ASC) 
            ) ENGINE = InnoDB";

        return $this->query($query);
    }

    public function create_data_table($ini_year, $end_year, $partitions_per_year)
    {
        $query = "CREATE TABLE IF NOT EXISTS data (
        `id` INT NOT NULL AUTO_INCREMENT ,
		`Partition` VARCHAR(20) NOT NULL ,
        `User` VARCHAR(64) NOT NULL ,
        `JobID` VARCHAR(64) NOT NULL,
        `JobName` VARCHAR(128) NOT NULL,
        `Submit` BIGINT(20) NOT NULL ,
        `Start` BIGINT(20) NOT NULL ,
        `End` BIGINT(20) NOT NULL ,
        `ElapsedRaw` BIGINT(20) NOT NULL ,
        `NodeList` VARCHAR(128) NOT NULL ,
        `NCPUS` BIGINT(20) NOT NULL,
        `CPUTimeRAW` BIGINT(20) NOT NULL,
        `MaxDiskRead` VARCHAR(64) NOT NULL,
        `MaxDiskWrite` VARCHAR(64) NOT NULL,
        `ReqTRES` VARCHAR(64) NOT NULL,
        `MaxVMSize` VARCHAR(64) NOT NULL,
        `State` VARCHAR(64) NOT NULL,
        `ExitCode` VARCHAR(64) NOT NULL,
        UNIQUE KEY (`JobID`, `Start`),
        UNIQUE KEY (`id`, `Start`)
	) ENGINE = InnoDB";

        if ($partitions_per_year == 0) {
            return $query;
        }
        $query .= " PARTITION BY RANGE(Start) ( ";

        $partition_number = 0;
        for ($year = $ini_year; $year <= $end_year; $year ++) {
            $first_timestamp_of_year = mktime(0, 0, 0, 1, 1, $year);
            $last_timestamp_of_year = mktime(23, 59, 59, 12, 31, $year);
            $increment = round(($last_timestamp_of_year - $first_timestamp_of_year) / $partitions_per_year);

            $step = $first_timestamp_of_year + $increment;
            for ($i = 0; $i < $partitions_per_year; $i ++) {
                if ($i == $partitions_per_year - 1) {
                    $step = $last_timestamp_of_year;
                }
                $comment = date("Y-m-d H:i:s", $step);
                $query .= " PARTITION p$partition_number VALUES LESS THAN ($step) COMMENT '$comment',";
                $partition_number ++;
                $step += $increment;
            }
        }
        $query .= " PARTITION p$partition_number VALUES LESS THAN MAXVALUE COMMENT 'catchall')";

        return $this->query($query);
    }
}
?>