<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Controller;

use Exception;
use sgae3\Helper\AjaxMessages;
use sgae3\Service\QueueService;
use sgae3\Service\AdminService;
use sgae3\Service\QueueAdminService;
use sgae3\Service\UserService;

class AdminController
{
    public function adminPage()
    {
        $queueService = new QueueService();
        $adminService = new AdminService();
        $queueAdminService = new QueueAdminService();
        $userService = new UserService();
        $queues = $queueService->getAllQueues();
        $admins = $adminService->getAdminUsernames();
        $queue_admins = $queueAdminService->getFullQueueAdmins();
        $users = $userService->getAllUsers();

        if(($log = file_get_contents($GLOBALS['app_logfile']))===false){
            $log="Unable to open log file";
        }

        require $GLOBALS['app_path'] . '/views/admin.phtml';
    }

    public function newQueue()
    {
        $newQueueName = null;
        $this->checkPost("queueName", $newQueueName, "Missing queue name.");

        $queueService = new QueueService();
        try {
            $queueService->newQueue($newQueueName);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("Queue '$newQueueName' saved");
    }

    public function toogleEnabledQueue()
    {
        $queueId = null;
        $this->checkPost("queueId", $queueId, "Missing queue.");

        $queueService = new QueueService();
        try {
            $queueService->toogleEnabled($queueId);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("Updated");
    }

    public function saveAdmins()
    {
        $admins_str = $_POST['admins'];
        $admins = array_map('trim',explode(",", $admins_str));
        $adminService = new AdminService();
        try {
            $adminService->saveAdmins($admins);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("Updated");
    }

    public function resetQueue(){
        $queueId = null;
        $this->checkPost("queueId", $queueId, "Missing queue.");

        $queueService = new QueueService();
        try {
            $queueService->reset($queueId);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("Updated");

    }

    public function saveQueueAdmins()
    {
        $this->checkPost("queue_id", $queue_id, "Missing queue.");
        $admins_str = $_POST['admins'];
        $queue_admins = array_map('trim',explode(",", $admins_str));
        $queueAdminService = new QueueAdminService();
        try {
            $queueAdminService->saveQueueAdmins($queue_id, $queue_admins);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("Updated");
    }

    private function checkPost($post_name, &$var_name, $msg)
    {
        if (!isset($_POST[$post_name]) || trim($_POST[$post_name]) == "") {
            AjaxMessages::ajaxError($msg);
        }
        $var_name = $_POST[$post_name];
    }

    public function runCron(): void {
        $command = "php ".$GLOBALS['app_path']."/cron/cron.php >> ".$GLOBALS['app_logfile']." 2>&1 &";
        exec($command, $output, $return_var);
        if ($return_var != 0) {
            trigger_error("ERROR: command '$command' ended with status $return_var and output: " . implode(",", $output), E_USER_ERROR);
            throw new Exception("Cron exception: see log for details");
        }
        AjaxMessages::ajaxSuccess("Cron launched in background: see log for details");
    }

    public function getCronStatus(): void {
        $cc = new CronController();
        if($cc->checkCronStatus()){
            AjaxMessages::ajaxSuccess("running");
        }
        AjaxMessages::ajaxSuccess("stopped");
    }

    public function newUser(){
        $newUsername = null;
        $this->checkPost("username", $newUsername, "Missing queue name.");
        $newPassword = null;
        $this->checkPost("password", $newPassword, "Missing queue name.");

        $userService = new UserService();
        try {
            $userService->newUser($newUsername, $newPassword);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess("User '$newUsername' saved");
    }

    public function deleteUser(){
        $user_id = null;
        $this->checkPost("user_id", $user_id, "Missing user id.");


        $userService = new UserService();
        try {
            $msg = $userService->deleteUser($user_id);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess($msg);
    }

    public function changeUserPassword(){
        $user_id = null;
        $this->checkPost("user_id", $user_id, "Missing user id.");
        $password = null;
        $this->checkPost("password", $password, "Missing user password.");


        $userService = new UserService();
        try {
            $msg = $userService->changeUserPassword($user_id, $password);
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
        AjaxMessages::ajaxSuccess($msg);
    }
}
