<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Helper;

use Exception;


class Squeue
{

    public function user_live_data(string $username, string $queue): array
    {

        $squeue = $GLOBALS['squeue_path'];
        $data = array();

        $command = "$squeue -p '$queue' -u '$username' -t 'running' -h | wc -l";
        $data['user_running_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -p '$queue' -u '$username' -t 'pending' -h | wc -l";
        $data['user_queued_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -h | wc -l";
        $data['user_cluster_total_jobs'] = $this->exec_squeue($command);

        return $data;
    }

    public function user_cluster_live_data(string $username): array
    {
        $squeue = $GLOBALS['squeue_path'];
        $data = array();

        $command = "$squeue -u '$username' -t 'running' -h | wc -l";
        $data['user_running_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -u '$username' -t 'pending' -h | wc -l";
        $data['user_queued_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -h | wc -l";
        $data['user_cluster_total_jobs'] = $this->exec_squeue($command);

        return $data;
    }

    public function queue_live_data(string $queue): array
    {
        $squeue = $GLOBALS['squeue_path'];
        $data = array();

        $command = "$squeue -p '$queue' -t 'running' -h | wc -l";
        $data['queue_running_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -h | wc -l";
        $data['queue_cluster_total_jobs'] = $this->exec_squeue($command);

        return $data;
    }

    public function cluster_live_data(): array
    {
        $squeue = $GLOBALS['squeue_path'];
        $data = array();

        $command = "$squeue -t 'running' -h | wc -l";
        $data['cluster_running_jobs'] = $this->exec_squeue($command);

        $command = "$squeue -h | wc -l";
        $data['cluster_total_jobs'] = $this->exec_squeue($command);

        return $data;
    }

    private function exec_squeue(string $command)
    {
        $output = array();
        $return_var = null;
        exec($command, $output, $return_var);
        if ($return_var != 0) {
            trigger_error("ERROR: command '$command' ended with status $return_var and output: " . implode(",", $output), E_USER_ERROR);
            throw new Exception("Squeue error: see log for details");
        }
        return $output[0];
    }
}
