<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Repository\UserRepository;

class DBLoginService implements LoginInterface
{

    /**
     * Does not apply: either $_SESSION['username'] is set or not
     * has been already checked at LoginController::checkLogged
     *
     * {@inheritdoc}
     * @see \sgae3\service\LoginInterface::login()
     */
    public function isLogged(string $username): bool
    {
        return false;
    }

    /**
     * Login against database with user and password
     *
     * {@inheritdoc}
     * @see \sgae3\service\LoginInterface::doLogin()
     */
    public function doLogin(string $username, string $password): bool
    {
        $userService = new UserService();

        try {
            if ($user = $userService->findUserByUsername($username)) {
                if ($this->rightCredentials($user, $password)) { 
                    $_SESSION['username'] = $username;
                    return true;
                } else {
                    return false;
                }
            } else {
                // User does not exist
                return false;
            }
        } catch (Exception $e) {
            trigger_error("Errors trying to login as user '$username': " . $e->getMessage(), E_USER_ERROR);
            throw new Exception('Login service unavailable.');
        }
    }

    private function rightCredentials(object $user, string $password)
    {
        try {
            $db_password = $user->getPassword();
            return password_verify($password, $db_password);
        } catch (Exception $e) {
            throw $e;
        }
    }

}
