<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Helper\Database;

class GraphsService
{

    const QUERY_ERROR = "Query error";
    const COMMON_COLUMNS = "
        IFNULL(MAX(IF(`State`='COMPLETED', `ElapsedRaw`, NULL)), 0) AS jobs_execution_time_longest,
        IFNULL(ROUND(AVG(IF(`State`= 'COMPLETED',`ElapsedRaw`, NULL))), 0) AS jobs_execution_time_average,
        IFNULL(MIN(IF(`State`='COMPLETED' AND `ElapsedRaw` > 0, `ElapsedRaw`, NULL)), 0) AS jobs_execution_time_shortest,
        SUM(IF(`State`= 'COMPLETED' AND `ElapsedRaw` <= 86400, 1, 0)) AS jobs_execution_time_1d,
        SUM(IF(`State`= 'COMPLETED' AND `ElapsedRaw` > 86400 AND `ElapsedRaw` <= 604800, 1, 0)) AS jobs_execution_time_7d,
        SUM(IF(`State`= 'COMPLETED' AND `ElapsedRaw` > 604800 AND `ElapsedRaw` <= 2678400, 1, 0)) AS jobs_execution_time_31d,
        SUM(IF(`State`= 'COMPLETED' AND `ElapsedRaw` > 2678400, 1, 0)) AS jobs_execution_time_xxd,
        SUM(IF(`State`= 'COMPLETED' , 1, 0)) AS jobs_success,
        SUM(IF(`State`= 'COMPLETED' , 0, 1)) AS jobs_failure,
        IFNULL(MAX(IF(`State`= 'COMPLETED', `MaxVMSize`, NULL)), 0) AS jobs_memory_maximum,
        IFNULL(ROUND(AVG(IF(`State`= 'COMPLETED', `MaxVMSize`, NULL))), 0) AS jobs_memory_average,
        IFNULL(MIN(IF(`State`= 'COMPLETED' AND `MaxVMSize`> 0,`MaxVMSize`, NULL)), 0) AS jobs_memory_minimum,
        SUM(IF(`State`= 'COMPLETED' AND `MaxVMSize` > 8589934592, 1, 0)) AS jobs_memory_xxgb,
        SUM(IF(`State`= 'COMPLETED' AND `MaxVMSize` <= 8589934592 AND `MaxVMSize` > 4294967296,1,0)) AS jobs_memory_8gb,
        SUM(IF(`State`= 'COMPLETED' AND `MaxVMSize` <= 4294967296 AND `MaxVMSize` > 1073741824,1,0)) AS jobs_memory_4gb,
        SUM(IF(`State`= 'COMPLETED' AND `MaxVMSize` <= 1073741824, 1, 0)) AS jobs_memory_1gb,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` >128, 1, 0)) AS Slots_x,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 64 AND `NCPUS` <= 128, 1, 0)) AS Slots_128,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 32 AND `NCPUS` <= 64, 1, 0)) AS Slots_64,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 16 AND `NCPUS` <= 32, 1, 0)) AS Slots_32,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 8 AND `NCPUS` <= 16, 1, 0)) AS Slots_16,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 4 AND `NCPUS` <= 8, 1, 0)) AS Slots_8,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 2 AND `NCPUS` <= 4, 1, 0)) AS Slots_4,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` > 1 AND `NCPUS` <= 2, 1, 0)) AS Slots_2,
        SUM(IF(`State`= 'COMPLETED' AND `NCPUS` <=1, 1, 0)) AS Slots_1";

    public function userGraphs($username, $queue, $from, $to)
    {
        $db = new database();

        $username = $db->real_escape_string($username);
        $queue = $db->real_escape_string($queue);
        $from = $db->real_escape_string($from);
        $to = $db->real_escape_string($to);

        $non_topten_where = "WHERE `User` = '$username' AND `Partition` = '$queue' AND `Start` >= '$from' AND `End` <= '$to' ";
        $topten_where = "$non_topten_where AND `State`= 'COMPLETED' ";

        // For "User queue usage"
        $query = "SELECT count(*) AS count 
            FROM data 
            WHERE `User` != '$username' AND `Partition` LIKE '$queue' AND `Start` >= '$from' AND `End` <= '$to'";
        
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $result = $db->fetch_one();
        $non_owner_queue_usage = $result['count'];
        $db->free();

        // For non Top-ten 
        $query = "SELECT " . self::COMMON_COLUMNS . ",
            COUNT(*) AS owner_queue_usage
            FROM data
            $non_topten_where";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $output = $db->fetch_one();
        $output['non_owner_queue_usage'] = $non_owner_queue_usage;


        //For top-ten node usage
        $query = "SELECT NodeList, Count(*) as count
            FROM data
            $topten_where
            GROUP BY NodeList ORDER BY count DESC LIMIT 10;";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['NodeList'];
            $values[] = $row['count'];
        }
        $output['user_topten_node_usage']['labels'] = $labels;
        $output['user_topten_node_usage']['values'] = $values;

        //For top-ten execution time
        $query = "SELECT ElapsedRaw
            FROM data
            $topten_where
            ORDER BY ElapsedRaw DESC LIMIT 10;
            ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $values = array();
        foreach ($db->fetch_all() as $row) {
            $values[] = round($row['ElapsedRaw'] / 3600, 2);
        }
        $output['user_topten_execution_time_abs']['values'] = $values;

        return $output;
    }

    public function queueGraphs($queue, $from, $to)
    {

        $db = new database();

        $queue = $db->real_escape_string($queue);
        $from = $db->real_escape_string($from);
        $to = $db->real_escape_string($to);

        $non_topten_where = "WHERE `Partition` = '$queue' AND `Start` >= '$from' AND `End` <= '$to' ";
        $topten_where = "$non_topten_where AND `State`= 'COMPLETED'";

        // For non Top-ten 
        $query = "SELECT" . self::COMMON_COLUMNS . "
            FROM data
            $non_topten_where";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $output = $db->fetch_one();

        //For top-ten node usage
        $query = "SELECT NodeList, Count(*) as count
                FROM data
                $topten_where
                GROUP BY NodeList ORDER BY count DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['NodeList'];
            $values[] = $row['count'];
        }
        $output['queue_topten_node_usage']['labels'] = $labels;
        $output['queue_topten_node_usage']['values'] = $values;

        //For top-ten user usage
        $query = "SELECT User, Count(*) as count
                FROM data
                $topten_where
                GROUP BY User ORDER BY count DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = $row['count'];
        }
        $output['queue_topten_users_usage']['labels'] = $labels;
        $output['queue_topten_users_usage']['values'] = $values;

        //For top-ten memory usage
        $query = "SELECT User, ROUND(AVG(MaxVMSize)) AS mem 
                FROM data
                $topten_where
                GROUP BY User ORDER BY mem DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['mem'] / (1024 * 1024), 2);
        }
        $output['queue_topten_mem_usage']['labels'] = $labels;
        $output['queue_topten_mem_usage']['values'] = $values;

        //For top-ten execution time usage
        $query = "SELECT User, ROUND(AVG(ElapsedRaw)) AS elapsed 
                FROM data
                $topten_where
                GROUP BY User ORDER BY elapsed DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['elapsed'] / 3600, 2);
        }
        $output['queue_topten_exec_time']['labels'] = $labels;
        $output['queue_topten_exec_time']['values'] = $values;

        //For top-ten execution time absolute
        $query = "SELECT User, ROUND(MAX(ElapsedRaw)) AS elapsed 
                FROM data
                $topten_where
                GROUP BY User ORDER BY elapsed DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['elapsed'] / 3600, 2);
        }
        $output['queue_topten_exec_time_abs']['labels'] = $labels;
        $output['queue_topten_exec_time_abs']['values'] = $values;



        return $output;
    }

    public function clusterGraphs($from, $to)
    {

        $db = new database();

        $from = $db->real_escape_string($from);
        $to = $db->real_escape_string($to);

        $non_topten_where = "WHERE `Partition` = `queue_name` AND `enabled_queue` = 1 AND `Start` >= '$from' AND `End` <= '$to' ";
        $topten_where = "$non_topten_where  AND `State`= 'COMPLETED'";

        // For non Top-ten 
        $query = "SELECT" . self::COMMON_COLUMNS . "
            FROM data, queue
            $non_topten_where";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $output = $db->fetch_one();

        //For top-ten node usage
        $query = "SELECT NodeList, Count(*) as count
                FROM data, queue
                $topten_where
                GROUP BY NodeList ORDER BY count DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['NodeList'];
            $values[] = $row['count'];
        }
        $output['queue_topten_node_usage']['labels'] = $labels;
        $output['queue_topten_node_usage']['values'] = $values;

        //For top-ten user usage
        $query = "SELECT User, Count(*) as count
                FROM data, queue
                $topten_where
                GROUP BY User ORDER BY count DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = $row['count'];
        }
        $output['queue_topten_users_usage']['labels'] = $labels;
        $output['queue_topten_users_usage']['values'] = $values;

        //For top-ten memory usage
        $query = "SELECT User, ROUND(AVG(MaxVMSize)) AS mem 
                FROM data, queue
                $topten_where
                GROUP BY User ORDER BY mem DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['mem'] / (1024 * 1024), 2);
        }
        $output['queue_topten_mem_usage']['labels'] = $labels;
        $output['queue_topten_mem_usage']['values'] = $values;

        //For top-ten execution time usage
        $query = "SELECT User, ROUND(AVG(ElapsedRaw)) AS elapsed 
                FROM data, queue
                $topten_where
                GROUP BY User ORDER BY elapsed DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['elapsed'] / 3600, 2);
        }
        $output['queue_topten_exec_time']['labels'] = $labels;
        $output['queue_topten_exec_time']['values'] = $values;

        //For top-ten execution time absolute
        $query = "SELECT User, ROUND(MAX(ElapsedRaw)) AS elapsed 
                FROM data, queue
                $topten_where
                GROUP BY User ORDER BY elapsed DESC LIMIT 10;
                ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['User'];
            $values[] = round($row['elapsed'] / 3600, 2);
        }
        $output['queue_topten_exec_time_abs']['labels'] = $labels;
        $output['queue_topten_exec_time_abs']['values'] = $values;



        return $output;
    }

    public function userClusterGraphs($username, $from, $to)
    {
        $db = new database();

        $username = $db->real_escape_string($username);
        $from = $db->real_escape_string($from);
        $to = $db->real_escape_string($to);

        $non_topten_where = "WHERE `User` = '$username' AND `Partition` = `queue_name` AND `enabled_queue` = 1 AND `Start` >= '$from' AND `End` <= '$to' ";
        $topten_where = "$non_topten_where AND `State`= 'COMPLETED'";

        // For "User queue usage"
        $query = "SELECT count(*) AS count 
            FROM data, queue 
            WHERE `User` != '$username' AND `Partition` = `queue_name` AND `enabled_queue` = 1 AND `Start` >= '$from' AND `End` <= '$to'";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $result = $db->fetch_one();
        $non_owner_queue_usage = $result['count'];
        $db->free();

        // For non Top-ten 
        $query = "SELECT" . self::COMMON_COLUMNS . ",
            COUNT(*) AS owner_queue_usage
            FROM data, queue
            $non_topten_where";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $output = $db->fetch_one();
        $output['non_owner_queue_usage'] = $non_owner_queue_usage;


        //For top-ten node usage
        $query = "SELECT NodeList, Count(*) as count
            FROM data, queue
            $topten_where
            GROUP BY NodeList ORDER BY count DESC LIMIT 10;
        ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $labels = array();
        $values = array();
        foreach ($db->fetch_all() as $row) {
            $labels[] = $row['NodeList'];
            $values[] = $row['count'];
        }
        $output['user_topten_node_usage']['labels'] = $labels;
        $output['user_topten_node_usage']['values'] = $values;

        //For top-ten execution time
        $query = "SELECT ElapsedRaw
            FROM data, queue
            $topten_where
            ORDER BY ElapsedRaw DESC LIMIT 10;
            ";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        $values = array();
        foreach ($db->fetch_all() as $row) {
            $values[] = round($row['ElapsedRaw'] / 3600, 2);
        }
        $output['user_topten_execution_time_abs']['values'] = $values;

        return $output;
    }

    public function yearlyGraphs()
    {

        $db = new database();

        //Get years
        $query = "SELECT DISTINCT FROM_UNIXTIME(`Submit`,'%Y') AS year FROM data";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $years = array();
        foreach ($db->fetch_all() as $row) {
            $years[] = $row['year'];
        }

        //Get queues
        $query = "SELECT DISTINCT `Partition` FROM data";
        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }
        $queues = array();
        foreach ($db->fetch_all() as $row) {
            $queues[] = $row['Partition'];
        }

        //Initialize a zero-filled output
        foreach ($queues as $queue) {
            $jobs_per_queue_values[$queue] = array();
            $total_time_per_queue_values[$queue] = array();
            $avg_mem_per_queue_values[$queue] = array();
            $avg_time_per_queue_values[$queue] = array();
            $avg_wait_time_per_queue_values[$queue] = array();
            foreach ($years as $year) {
                $jobs_per_queue_values[$queue][$year] = 0;
                $total_time_per_queue_values[$queue][$year] = 0;
                $avg_mem_per_queue_values[$queue][$year] = 0;
                $avg_time_per_queue_values[$queue][$year] = 0;
                $avg_wait_time_per_queue_values[$queue][$year] = 0;
            }
        }

        $query = "SELECT FROM_UNIXTIME(`Submit`,'%Y') AS year, `Partition`,  
            COUNT(*) AS jobs_success,
            IFNULL(ROUND(AVG(`MaxVMSize`)),0) AS jobs_memory_average,
            IFNULL(ROUND(AVG(`ElapsedRaw`)),0) AS jobs_execution_time_average,
            IFNULL(SUM(`ElapsedRaw`),0) AS jobs_execution_total_time,
            IFNULL(ROUND(AVG(`Start` - `Submit`)),0) AS jobs_execution_waiting_time_average
            FROM data
            WHERE `State`= 'COMPLETED'
            GROUP BY `Partition`, year
            ORDER BY year";

        if (!$db->query($query)) {
            throw new Exception(self::QUERY_ERROR);
        }

        foreach ($db->fetch_all() as $row) {
            $year = $row['year'];
            $Partition = $row['Partition'];
            $jobs_per_queue_values[$Partition][$year] = $row['jobs_success'];
            $total_time_per_queue_values[$Partition][$year] = round($row['jobs_execution_total_time'] / 3600, 2);
            $avg_mem_per_queue_values[$Partition][$year] = round($row['jobs_memory_average'] / (1024 * 1024), 2);
            $avg_time_per_queue_values[$Partition][$year] = round($row['jobs_execution_time_average'] / 3600, 2);
            $avg_wait_time_per_queue_values[$Partition][$year] = round($row['jobs_execution_waiting_time_average'] / 60, 2);
        }

        $output["years"] = $years;
        $output["partitions"] = $queues;
        $output["yearly_jobs_per_queue_data"]["values"] = $jobs_per_queue_values;
        $output["yearly_jobs_execution_total_time_data"]["values"] = $total_time_per_queue_values;
        $output["yearly_avg_mem_per_queue_data"]["values"] = $avg_mem_per_queue_values;
        $output["yearly_avg_time_per_queue_data"]["values"] =  $avg_time_per_queue_values;
        $output["yearly_avg_wait_time_per_queue_data"]["values"] = $avg_wait_time_per_queue_values;
        return $output;
    }
}
