/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Controls new render or update of charts
var rendered = false;

$(function () {

    $("#nav_user").addClass("active");

    $("#user_graphs").hide();

    $("#date_range").change(function () {
        $("#start_time").prop('required', true);
        $("#end_time").prop('required', true);
    })
    $("#full_year").change(function () {
        $("#start_time").prop('required', false);
        $("#end_time").prop('required', false);
    })
    $("#submit").click(submit_form)
})

function submit_form() {

    // Delegated Browser/HTML5 validation
    if (!$("#queue_select")[0].validity.valid) {
        display_error("Invalid queue value");
        return false;
    }
    if (!$("#year_select")[0].validity.valid) {
        display_error("Invalid year value");
        return false;
    }
    if (!$("#start_time")[0].validity.valid) {
        display_error("Invalid start time value");
        return false;
    }
    if (!$("#end_time")[0].validity.valid) {
        display_error("Invalid end time value");
        return false;
    }

    $.blockUI({
        message: '<h4>Loading data, please wait</h4><img src="./static/images/ajax-loader.gif"/>',
        css: {
            width: 'auto',
            left: '45%',
            padding: '15px',
        },
    });

    $("#user_graphs").show();
    
    // Ajax
    $.ajax({
        method: "POST",
        url: "user/userGraphs",
        data: $("#user_form").serialize(),
        success: function (data) {
            try {
                var json = JSON.parse(data);
                if (json['status'] == "error") {
                    display_error(json['error'])
                    $.unblockUI();
                } else {
                    prepare_data_for_user_graphs(json['data'])
                    if (rendered) {
                        update_graphs(user_charts)
                    } else {
                        rendered = true
                        draw_graphs(user_charts)
                    }
                }
                $.unblockUI();
            } catch (e) {
                display_error("An error has occurred")
                $.unblockUI();
                console.log(e)
            }
        },
        error: [ajax_error, $.unblockUI]
    });
    return false;
}

